from pathlib import Path
from tqdm.auto import tqdm
import numpy as np
import pandas as pd

from transformers import AutoModel, AutoTokenizer, DataCollatorWithPadding, \
    AutoModelForSequenceClassification, TrainingArguments, Trainer
from datasets import load_metric, load_dataset, DatasetDict, concatenate_datasets
import torch
import logging

logging.basicConfig(level=logging.INFO)

DEVICE = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
#PATH_TO_DATA = Path('../data/')
#PATH = '../input/detecting-generated-scientific-papers'
INDEX_COL_NAME = 'id'
INPUT_COL_NAME = 'text'
TARGET_COL_NAME = 'fake'

#logging.info("read train.csv")
#Reading and briefly exploring the available data
#train_df = pd.read_csv(PATH_TO_DATA / "synscipass_dataset_train.csv", index_col=INDEX_COL_NAME)
logging.info("loading new dataset")
new_dataset = load_dataset(
    'csv',
    data_files={
        'train': '../data/synscipass_dataset_train.csv', #I'm adding the files from the data folder
        'validation': '../data/synscipass_dataset_validation.csv',
        'test': '../data/synscipass_dataset_test.csv'
    }
)
#original path: ../input/detectinggeneratedscientificaugmented/real_and_fake_passages_dataset_test.csv

# new_dataset = new_dataset.remove_columns(['Unnamed: 0', 'type', 'model'])
logging.info("renaming columns")
new_dataset = new_dataset.rename_column('passages', 'text')

####### DEFINING THE MODEL #######
logging.info("defining model")

tokenizer = AutoTokenizer.from_pretrained("microsoft/deberta-v3-large")
model = AutoModelForSequenceClassification.from_pretrained("microsoft/deberta-v3-large", num_labels=4)
model.to(DEVICE)

logging.info("preprocess function")
def preprocess_function(examples):
    return tokenizer(examples["text"], truncation=True, max_length=256)

tool_labels = {
    "generate": 0,
    "paraphrase": 1,
    "real": 2,
    "translate": 3 
}

logging.info("preprocess labels")
def preprocess_label(examples):
    examples['label'] = [tool_labels[tool] for tool in examples['tool']]
    return tokenizer(examples["text"], truncation=True, max_length=256)

# tokenized_dataset = new_dataset.filter(lambda example: example['tool'] == 'unknown' or example['tool'] == 'real').map(preprocess_function, 
# batched=True).rename_column("fake", "label")
logging.info("making tokenized dataset")
tokenized_dataset = new_dataset.filter(lambda example: example['model'] != 'GPT-2-arxiv_generate').map(preprocess_label, 
                                                                                                       batched=True)
tokenized_train = tokenized_dataset["train"]
tokenized_val = tokenized_dataset["validation"]
logging.info("collating data and loading model")
data_collator = DataCollatorWithPadding(tokenizer=tokenizer)


def compute_metrics(eval_preds):
    precision = load_metric("precision")
    recall = load_metric("recall")
    f1 = load_metric("f1")
    logits, labels = eval_preds
    predictions = np.argmax(logits, axis=-1)
    return {
        **f1.compute(predictions=predictions, references=labels, average='micro'),
        **precision.compute(predictions=predictions, references=labels, average='micro'),
        **recall.compute(predictions=predictions, references=labels, average='micro')
    }
if __name__ == "__main__":
    training_args = TrainingArguments(
        "deberta-v3-large-finetuned-test_fp16",
        learning_rate=6e-6,
        per_device_train_batch_size=4,
        per_device_eval_batch_size=4,
        num_train_epochs=1,
        weight_decay=0.01,
        warmup_steps=50,
        evaluation_strategy="epoch",
        save_strategy="epoch",
        save_total_limit=1,
        push_to_hub=True,
        fp16=True # switch off if not using GPU
    )

    trainer = Trainer(
        model=model,
        args=training_args,
        train_dataset=tokenized_dataset["train"],
        eval_dataset=tokenized_dataset["validation"],
        tokenizer=tokenizer,
        data_collator=data_collator,
        compute_metrics=compute_metrics,
    )
    logging.info("evaluating on validation set")
    trainer.evaluate(tokenized_dataset["validation"].select(range(32)))
    logging.info("training")
    trainer.train()
    logging.info("evaluating on test set")
    trainer.evaluate(
        tokenized_dataset["test"]
    )
    logging.info("pushing model to hub!")
    trainer.push_to_hub(commit_message="Training complete", tags="text-classification")
    logging.info("done!")